@@ -10,22 +10,21 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-from tests import unittest
+from tests import unittest, RawResponse
 import datetime
-import collections
 
 from dateutil.tz import tzutc
 from nose.tools import assert_equal
 
 from botocore import parsers
 from botocore import model
-from botocore.compat import json
+from botocore.compat import json, MutableMapping
 
 
 # HTTP responses will typically return a custom HTTP
 # dict.  We want to ensure we're able to work with any
 # kind of mutable mapping implementation.
-class CustomHeaderDict(collections.MutableMapping):
+class CustomHeaderDict(MutableMapping):
     def __init__(self, original_dict):
         self._d = original_dict
 
@@ -225,6 +224,35 @@ class TestResponseMetadataParsed(unittest.TestCase):
                                           'HTTPStatusCode': 200,
                                           'HTTPHeaders': headers}})
 
+    def test_response_no_initial_event_stream(self):
+        parser = parsers.JSONParser()
+        output_shape = model.StructureShape(
+            'OutputShape',
+            {
+                'type': 'structure',
+                'members': {
+                    'Payload': {'shape': 'Payload'}
+                }
+            },
+            model.ShapeResolver({
+                'Payload': {
+                    'type': 'structure',
+                    'members': [],
+                    'eventstream': True
+                }
+            })
+        )
+        with self.assertRaises(parsers.ResponseParserError):
+            response_dict = {
+                'status_code': 200,
+                'headers': {},
+                'body': RawResponse(b''),
+                'context': {
+                    'operation_name': 'TestOperation'
+                }
+            }
+            parser.parse(response_dict, output_shape)
+
     def test_metadata_always_exists_for_json(self):
         # ResponseMetadata is used for more than just the request id. It
         # should always get populated, even if the request doesn't seem to
@@ -569,8 +597,8 @@ class TestHandlesInvalidXMLResponses(unittest.TestCase):
         parser = parsers.QueryParser()
         output_shape = None
         # The XML body should be in the error message.
-        with self.assertRaisesRegexp(parsers.ResponseParserError,
-                                     '<DeleteTagsResponse'):
+        with self.assertRaisesRegex(parsers.ResponseParserError,
+                                    '<DeleteTagsResponse'):
             parser.parse(
                 {'body': invalid_xml, 'headers': {}, 'status_code': 200},
                 output_shape)
@@ -626,6 +654,7 @@ class TestEventStreamParsers(unittest.TestCase):
                     'EventB': {'shape': 'EventBStructure'},
                     'EventC': {'shape': 'EventCStructure'},
                     'EventD': {'shape': 'EventDStructure'},
+                    'EventException': {'shape': 'ExceptionShape'},
                 }
             },
             model.ShapeResolver({
@@ -684,7 +713,14 @@ class TestEventStreamParsers(unittest.TestCase):
                 },
                 'BlobShape': {'type': 'blob'},
                 'StringShape': {'type': 'string'},
-                'IntShape': {'type': 'integer'}
+                'IntShape': {'type': 'integer'},
+                'ExceptionShape': {
+                    'exception': True,
+                    'type': 'structure',
+                    'members': {
+                        'message': {'shape': 'StringShape'}
+                    }
+                },
             })
         )
 
@@ -767,7 +803,7 @@ class TestEventStreamParsers(unittest.TestCase):
         self.assertEqual(parsed, expected)
 
     def test_parses_error_event(self):
-        error_code = 'client/SomeError',
+        error_code = 'client/SomeError'
         error_message = 'You did something wrong'
         headers = {
             ':message-type': 'error',
@@ -784,6 +820,23 @@ class TestEventStreamParsers(unittest.TestCase):
         }
         self.assertEqual(parsed, expected)
 
+    def test_parses_exception_event(self):
+        self.parser = parsers.EventStreamJSONParser()
+        error_code = 'EventException'
+        headers = {
+            ':message-type': 'exception',
+            ':exception-type': error_code,
+        }
+        body = b'{"message": "You did something wrong"}'
+        parsed = self.parse_event(headers, body, status_code=400)
+        expected = {
+            'Error': {
+                'Code': error_code,
+                'Message': 'You did something wrong'
+            }
+        }
+        self.assertEqual(parsed, expected)
+
     def test_parses_event_json(self):
         self.parser = parsers.EventStreamJSONParser()
         headers = {':event-type': 'EventD'}
@@ -809,6 +862,21 @@ class TestParseErrorResponses(unittest.TestCase):
     # this into the shared protocol tests in the future,
     # so consolidating them into a single class will make
     # this easier.
+    def setUp(self):
+        self.error_shape = model.StructureShape(
+            'ErrorShape',
+            {
+                'type': 'structure',
+                'exception': True,
+                'members': {
+                    'ModeledField': {
+                        'shape': 'StringType',
+                    }
+                }
+            },
+            model.ShapeResolver({'StringType': {'type': 'string'}})
+        )
+
     def test_response_metadata_errors_for_json_protocol(self):
         parser = parsers.JSONParser()
         response = {
@@ -1112,6 +1180,103 @@ class TestParseErrorResponses(unittest.TestCase):
         self.assertEqual(parsed['Error'], {'Message': 'Access denied',
                                            'Code': 'AccessDeniedException'})
 
+    def test_can_parse_rest_json_modeled_fields(self):
+        body = (
+            b'{"ModeledField":"Some modeled field",'
+            b'"Message":"Some message"}'
+        )
+        parser = parsers.RestJSONParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_rest_xml_modeled_fields(self):
+        parser = parsers.RestXMLParser()
+        body = (
+            b'<?xml version="1.0"?>\n<ErrorResponse xmlns="http://foo.bar">'
+            b'<Error><Type>Sender</Type><Code>NoSuchDistribution</Code>'
+            b'<Message>The specified distribution does not exist.</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error>'
+            b'</ErrorResponse>'
+        )
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_ec2_modeled_fields(self):
+        body = (
+            b'<Response><Errors><Error>'
+            b'<Code>ExceptionShape</Code>'
+            b'<Message>Foo message</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error></Errors></Response>'
+        )
+        parser = parsers.EC2QueryParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_query_modeled_fields(self):
+        parser = parsers.QueryParser()
+        body = (
+            b'<?xml version="1.0"?>\n<ErrorResponse xmlns="http://foo.bar">'
+            b'<Error><Type>Sender</Type><Code>SomeCode</Code>'
+            b'<Message>A message</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error>'
+            b'</ErrorResponse>'
+        )
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_json_modeled_fields(self):
+        body = (
+            b'{"ModeledField":"Some modeled field",'
+            b'"Message":"Some message",'
+            b'"__type": "Prefix#SomeError"}'
+        )
+        parser = parsers.JSONParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
     def test_can_parse_route53_with_missing_message(self):
         # The message isn't always in the XML response (or even the headers).
         # We should be able to handle this gracefully and still at least
